################################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################

import transaction
import logging

LOG = logging.getLogger('haufe.monitoring')

def initialize(context): 
    """$ initialize haufe.monitoring """
    from haufe.monitoring.controlpanel import ControlPanel, ControlPanelId

    LOG.info('Initializing control panel')
    cp = context._ProductContext__app.Control_Panel
    if not ControlPanelId in cp.objectIds():
        monitor = ControlPanel()
        cp._setObject(ControlPanelId, monitor)
        transaction.commit()

    monitor = cp[ControlPanelId]

    LOG.info('attempt to register this Zope Instance with ControlPanel')
    from ZODB.POSException import ConflictError
    retry_count = 5
    try:
        while retry_count > 0:
            try:
                monitor.registerZopeInstance()
                retry_count = 0
            except ConflictError:
                retry_count -= 1
                LOG.warn('ConflictErorr while registering Zope Instance')
                if retry_count > 0: 
                    continue
                raise # retry action
        LOG.info('Zope instance successfully registered')
        transaction.commit()
 
    except:
         LOG.error('Failed to register Zope instance', exc_info=True)

    LOG.info('Starting GIL monitor')
    import gil_monitor
    gil_monitor.start()

    # reconfigure garbage collector
    #  generation 0 GC at "(allocated - freed) == 7.000"; analyse 7.000 objects
    #  generation 1 GC at "(allocated - freed) == 140.000"; analyse 140.000 objects
    #  generation 2 GC at "(allocated - freed) == 1.400.000"; analyse all objects

    LOG.info('reconfigured garbage collector thresholds')
    import gc
    gc.set_threshold(7000, 20, 10)




################################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################

import os

class ProcStatus:
    """
    determine Process Status for a given or current process
    originaly taken from work by Nils Weinzierl (memory profiling in Haufe HRS environment)

    see also 

    http://mail.python.org/pipermail/python-list/2004-June/266257.html
    How to get process info from python

    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/286222
    Memory usage. 
    """
    
    def __init__(self,procid):
        self.name = 'status'
        self.format = ('f', 5, 1024)
        self.f = file('/proc/%s/status' % procid, 'r', 0)
        self.vars = ('SleepAVG', 'VmPeak', 'Cached', 'MemFree','Threads','VmData')
        
    def extract(self, vars=None):
        self.val = {}
        if not vars:
            vars = self.vars
        self.f.seek(0)
        for line in self.f.readlines():
            l = line.split(':')
            l[1] = l[1].strip()
            if l[0] in vars:
                self.val[l[0]] = l[1]
        return self.val    

def getLoad():
    load = file('/proc/loadavg').read().split()[0]
    return load

def getVmDataSize():
    """returns the virtual Memory needed for the process with the id pid """

    pid = os.getpid()
    try:
        vals = ProcStatus(pid).extract()
        return int(vals['VmData'].replace(' kB',''))
    except:
        return -1

if __name__ == '__main__':
    print getLoad()
    print getVmDataSize()



###############################################################
# haufe.monitoring
# (C) 2009, Haufe Mediengruppe, Written by Andreas Jung
################################################################

import os
import urllib
import urllib2
import demjson
import socket
import logging

from Products.Five.browser import BrowserView
from Globals import InitializeClass

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from haufe.monitoring.controlpanel import ControlPanelId
from haufe.monitoring.network import isRequestAllowed

LOG = logging.getLogger('haufe.monitoring')

class MonitorView(BrowserView):

    template = ViewPageTemplateFile('monitor.pt')

    def __call__(self):
        return self.template()

    def getServers(self):
        return self.context.Control_Panel[ControlPanelId].getServers()

    def getStats(self):

        stats = dict()
        for server in self.getServers():

            # the remote URL for obtaining the system stats
            url = 'http://%s:%d/@@system_information' % (server.hostname, server.port)

            stats[(server.hostname, server.port)] = None
            try:
                socket.setdefaulttimeout(5)
                json_data = urllib2.urlopen(url).read()
                stats[(server.hostname, server.port)] = demjson.decode(json_data)
                socket.setdefaulttimeout(None)
            except urllib2.URLError, e:
                socket.setdefaulttimeout(None)
                continue

        return stats

    def remove_hosts(self):

        for hostSpec in self.request.get('hosts', []):
            host, port = hostSpec.split(':')
            self.context.Control_Panel[ControlPanelId].unregisterZopeInstance(host, int(port))
        return self.request.response.redirect('/@@monitor')

    def restart(self):
        """ Restart Zope instances """

        for hostSpec in self.request.get('hosts', []):
            host, port = hostSpec.split(':')
            url = 'http://%s:%s/Control_Panel/manage_restart' % (host, port)
            request = urllib2.Request(url)
            response = urllib2.urlopen(request)
            answer = response.read()

    def shutdown(self):
        """ Shutdown Zope instances """

        for hostSpec in self.request.get('hosts', []):
            host, port = hostSpec.split(':')
            url = 'http://%s:%s/Control_Panel/manage_shutdown' % (host, port)
            request = urllib2.Request(url)
            response = urllib2.urlopen(request)
            answer = response.read()

InitializeClass(MonitorView)
